#include <TMath.h>
#include <TRandom.h>
#include <iostream>

#include "MySignalTrace.h"



//________________create a signal trace___________________________________________________________________________________________________________________________________________
void MySignalTrace::createTrace()
{
	//write the offset to the whole trace//
	for (size_t i=0; i<length;++i)
		puls[i] = offset*mV2Bits();

	for (size_t iSig=0;iSig<sigs.size();++iSig)
	{
		//prepare the width//
		double fwhm = sigs[iSig].Width()*gs;
		double fwhm2 = sigs[iSig].Assymetric()*fwhm;
		double A  = 2.*((fwhm /2.35)*(fwhm /2.35));
		double A2 = 2.*((fwhm2/2.35)*(fwhm2/2.35));

		//add signal to trace//
		int start = sigs[iSig].Pos()*gs - 5* sigs[iSig].Width()*gs;
		size_t stop = sigs[iSig].Pos()*gs + 5* sigs[iSig].Width()*gs;
		start = (start >= 0)?start:0;
		stop = (stop > length)?length:stop;
		for (size_t i=start; i<stop;++i)
		{
			////leading edge//
			if (i <= sigs[iSig].Pos())
			{
				long tmp = (sigs[iSig].Height()*mV2Bits() * exp(-(((double)i-sigs[iSig].Pos()*gs)*((double)i-sigs[iSig].Pos()*gs))/A) );
				tmp += puls[i];

				puls[i] = checkBoundries(tmp);
			}
			//trailing edge with ringin//
			else if (i > sigs[iSig].Pos()*gs)
			{
				long tmp = (sigs[iSig].Height()*mV2Bits() * exp(-(((double)i-sigs[iSig].Pos()*gs)*((double)i-sigs[iSig].Pos()*gs))/A2) );
				tmp += ( sigs[iSig].Ringing()*sigs[iSig].Height()*mV2Bits() * -exp(-(((double)i-(sigs[iSig].Pos()*gs+fwhm2))*((double)i-(sigs[iSig].Pos()*gs+fwhm2)))/A2) );
				tmp += puls[i];

				puls[i] = checkBoundries(tmp);
			}
		}
	}

	if (noise) addNoiseToTrace();
	sigs.clear();
}

//________________check the Boundries__________________________________________________________
long MySignalTrace::checkBoundries(long value)
{
	long retval = value;
	
	if (tenBit)
	{
		if (value > 511) retval = 511;
		if (value < -512) retval = -512;
	}
	else
	{
		if (value > 127) retval = 127;
		if (value < -128) retval = -128;
	}

	return retval;
}



//________________add noise to puls___________________________________________________________________________________________________________________________________________
void MySignalTrace::addNoiseToTrace()
{
	double maxLevel = maxLevel_mV * mV2Bits();
	//create nbroffreq different frequenzes with random level and phase offset//
	Noise * n = new Noise[nbrFreq];
	for (size_t freq=0;freq < nbrFreq;++freq)
	{
		n[freq].freq_MHz = gRandom->Rndm() * (maxFreq_MHz-minFreq_MHz) + minFreq_MHz;
		n[freq].level = gRandom->Rndm() * maxLevel;
		n[freq].phOff = gRandom->Rndm() * 2.* TMath::Pi();
	}
	
	for (size_t i=0; i<length; ++i)
	{
		double add = 0;
		for (size_t freq=0;freq < nbrFreq;++freq)
			add += sin((double)i *2.*TMath::Pi()* n[freq].freq_MHz/1000. + n[freq].phOff)*n[freq].level;

		long tmp = puls[i]+add;
		
		puls[i] = checkBoundries(tmp);
	}
	delete [] n;
}
