#ifndef __MySignalTrace_H_
#define __MySignalTrace_H_

#include <vector>

class MySignal
{
public:
	MySignal():
	  height(0),
	  pos(0),
	  width(0),
	  ringing(0)	{}
	
	MySignal(double Position, double Height):
	  height(Height),
	  pos(Position),
	  width(5),
	  ringing(0.07),
	  assymetric(3)	{}

	~MySignal()		{}
	
	double Height() const		{return height;}
	double Pos() const			{return pos;}
	double Width() const		{return width;}
	double Ringing() const		{return ringing;}
	double Assymetric() const	{return assymetric;}
	

private:
	double height;
	double pos;
	double width;
	double ringing;
	double assymetric;
};



struct Noise{double phOff; double freq_MHz; double level;};
typedef std::vector<MySignal> sigVec;

class MySignalTrace
{
public:
	MySignalTrace():
	  puls(0),
	  minFreq_MHz(50),
	  maxFreq_MHz(300),
	  maxLevel_mV(2),
	  nbrFreq(50),
	  fullscale(1000),
	  gs(1),
	  offset(-400),
	  tenBit(false),
	  length(300)
								{puls = new long[length];setFsInBits();}

	~MySignalTrace()			{delete [] puls;}

protected:
	MySignalTrace& operator=(const MySignalTrace& in)	{return *this;}
	MySignalTrace(const MySignalTrace& in)				{}

public:
	void createTrace();
	void addSignal(double pos, double height)	{sigs.push_back(MySignal(pos,height));}
	void addNoise(bool in)						{noise = in;}

	void GS(double in)			{gs=in;}
	double SampInt() const		{return (1./gs);}
	double GS() const			{return gs;}

	void TenBit(bool in)		{tenBit=in; setFsInBits();}
	bool TenBit()const 			{return tenBit;}

	void Fullscale(double in)	{fullscale=in;}
	double Fullscale() const	{return fullscale;}

	double Bits2mV() const		{return (fullscale/fsInBits);}
	double mV2Bits() const		{return (fsInBits/fullscale);}
	
	double Offset() const		{return static_cast<int>(offset*mV2Bits());}
	void Offset(double in)		{offset = in;}

	size_t Length()	const		{return length;}
	void Length(size_t in)		{delete [] puls; puls = new long[in]; length = in;}
	
	const long operator[](size_t idx)const	{return *(puls+idx);}

	long* SigTrace() const		{return puls;}


private:
	void preparePuls(long offsetInBits);
	long checkBoundries(long value);
	void createPuls(double x, double h);
	void addNoiseToTrace();
	void setFsInBits()						{fsInBits=(tenBit)?1024.:256;}

	double offset;
	size_t length;
	double fullscale;
	double gs;
	bool tenBit;
	double fsInBits;

	bool noise;

	size_t nbrFreq;
	double minFreq_MHz;
	double maxFreq_MHz;
	double maxLevel_mV;

	long* puls;

	sigVec sigs;
};

#endif