#include "MyZeroSuppressor.h"

#include "../MyChannel/MyChannel.h"
#include "../MyPuls/MyPuls.h"

//______________________________________________________________________________________________________________________
template <typename T>
void suppressZerosImpl(MyChannel * ch)
{
	MyPuls * pu = 0;
		
	//reset the flag//
	bool firsttime = true;
	
	//get some thing we need to know for the zero suppression//
	long Datalength = ch->getRawDataLength();
	long baseline = ch->getBaseline();
	long noiselevel = ch->getNoise();
	long backsize = ch->getBackSize();
	long stepsize = ch->getStepSize();

	T * Start = static_cast<T *> (ch->getRawData());
	T * End = Start + Datalength-stepsize;
	T * Iter = Start;
	long rempos=0;

	while(Iter < End)
	{
		//--look if peak comes out of noiselevel--//
		if ((abs((*(Iter  ))-baseline) > noiselevel)  ||
			(abs((*(Iter+3))-baseline) > noiselevel))
		{
			//--if we are out of the noise for the first time then--//
			if (firsttime)
			{
				//--if it was the first time we went out of noise--//
				firsttime=false;

				//--if the remembered position is further away than backsize then--//
				//--go back backsize--//
				//--otherwise only go back i-rempos--//
				//--and remember position--//

				long i = Iter - Start;
				long back  = ((i-rempos) > backsize)? backsize : (i-rempos);
				
				//--if the remembered position is not further away than backsize--//
				//--add puls only if there is no pulse, that means that rempos is 0.--//
				//--otherwise just extend the previous pulse--//
				if (( rempos == 0) || ((i-rempos) > backsize) )
				{
					pu = ch->addPuls();
					pu->setIndexToFirstPoint((i-back));
					pu->setData((char*)(Iter-back));
					//--we need to append the length to "back+1" because the length				--//
					//--of the array is one bigger than the index								--//
					//--example: the pulse goes out of the noise level at index 3				--//
					//--   this is the 4th point. Now if we have a backsize of 50				--//
					//--   then back will be 3: this is ok for the timestamp, which is 0 and	--//
					//--   the pointer, which will point to data at index 0						--//
					//--   but we already have a length of 4 Points, which is "back+1"			--//
					pu->appendLength(back+1);
				}
				else
				{
					pu->appendLength(back);
				}

			}
			//--append next step--//
			pu->appendLength(stepsize);
			Iter+=stepsize;
		}
		else
		{
			//--if it is the firstime we are inside the noise again,--//
			//--remember the last point of the previous pulse--//
			if (!firsttime)
				rempos = Iter - Start;


			//--we are insinde the noise reset the firstime flag and advance 2 steps--//
			firsttime=true;
			Iter+=2;
		}
	}

	//--if the loop quitted but there was still a puls, record everything 'til the end of the range--//
	if (!firsttime)
		pu->appendLength(Datalength - (Iter - Start) -1);
}

//###########################################8bit stuff################################################################
//______________________________________________________________________________________________________________________
void MyZeroSuppressor8Bit::suppressZeros(MyChannel* ch) const
{
	suppressZerosImpl<char>(ch);
}

//##########################################16bit stuff#################################################################
//______________________________________________________________________________________________________________________
void MyZeroSuppressor16Bit::suppressZeros(MyChannel* ch) const
{
	suppressZerosImpl<short>(ch);
}

//##########################################32bit stuff#################################################################
//______________________________________________________________________________________________________________________
void MyZeroSuppressor32Bit::suppressZeros(MyChannel* ch) const
{
	suppressZerosImpl<long>(ch);
}
