class MyEvent;
class MyEventViewer;

void BuildEvent(MyEvent* ev, int evNbr)
{
	//clear the event first//
	ev->clear();

	//set the signaltrace//
	MySignalTrace st;
	st.Length(6e3);
	st.addNoise(true);

	//give event the infos that it needs//
	ev->setNbrSamples(st.Length());
	ev->setSampleInterval(1e-9);

	MyZeroSuppressor *zs = new MyZeroSuppressor32Bit();

	//for Each Channel of this event, create a Signal Trace with up to 10 Signals in it//
	for (size_t chan=0;chan <ev->getNbrOfChannels();++chan)
	{
		//size_t nSigs= gRandom->Uniform(10);
		size_t nSigs= 10-evNbr;
		for (size_t i=0;i<nSigs;++i)
		{
			double pos = gRandom->Rndm()*st.Length();
			double height = gRandom->Rndm()*st.Fullscale();
			st.addSignal(pos,height);
		}
		st.createTrace();
		ev->getChannel(chan)->copyRawData((char*)st.SigTrace(),st.Length());
		ev->getChannel(chan)->setBaseline(st.Offset());
		ev->getChannel(chan)->setOffset((st.Offset())*(st.Bits2mV()));
		ev->getChannel(chan)->setNoise(100*st.mV2Bits());
		ev->getChannel(chan)->setBackSize(30);
		ev->getChannel(chan)->setStepSize(50);
		ev->getChannel(chan)->setVertGain(st.Bits2mV());
		ev->getChannel(chan)->setFullScale(st.Fullscale());
		//Analyze the Signal Traces and create the Puls Array for each channel//
		zs->suppressZeros(ev->getChannel(chan));
	}

	delete zs;
}

void drawEvent(MyEvent * event, MyEventViewer * ew)
{
	//cout << "Event: "<<endl;
	//cout << " nChans: " << event->getNbrOfChannels()<<endl;
	//for (int chan=0; chan < event->getNbrOfChannels();++chan)
	//{
	//	MyChannel * Chan = event->getChannel(chan);

	//	cout << " Channel: "<< chan+1<<endl;
	//	cout << "  nPulses: " << Chan->getNbrPulses()<<endl;
	//	for (int puls=0; puls < Chan->getNbrPulses();++puls)
	//	{
	//		MyPuls * Puls = Chan->getPuls(puls);

	//		cout << "  Puls: "<<puls+1<<endl;
	//		cout << "   Length: "<<Puls->getLength()<<endl;
	//	}
	//}
	ew->view(event);
}


void writeRootFile(MyEventViewer * ew)
{
	//create a tree in the file test.root//
	TFile f("Test.root","recreate");
	TTree *T = new TTree("T","Testing 1 2 3");

	//create an event with random nbr of up to 24 Channels//
	//long nbrChans = gRandom->Uniform(24);
	long nbrChans = 4;	//restrict it to 4 Channels
	cout << "Creating Event with "<<nbrChans<<" Channels"<<endl;
	MyEvent *event = new MyEvent(nbrChans);
	event->setNbrBytes(4);	//we are currently using an int array

	//create a Branch that will hold the event
	T->Branch("event","MyEvent",&event,8000,2);

	//create a random Nbr of Event with up to 10 Events//
	//int nEvents = gRandom->Uniform(100);
	int nEvents = 4;	//restrict it to 4 events
	cout << "Filling Tree with "<<nEvents<<" Events"<<endl;
	for (size_t i=0;i<nEvents;++i)
	{
		cout << "working on Event "<< i+1 <<endl;
		BuildEvent(event,i);
		 //if eventvwr doesn't exist create it
		if (!ew) ew = new MyEventViewer(event,"Writer");
		drawEvent(event,ew);
		cout << "now filling tree"<<endl;
		T->Fill();
	}
	//T->Print();
	T->Write();
}


void readRootFile( MyEventViewer* ew) 
{
	TFile f("Test.root");
	TTree *T = reinterpret_cast<TTree*> (f.Get("T"));
	MyEvent *event = 0;
	T->SetBranchAddress("event", &event);
	size_t nEvents = static_cast<size_t>(T->GetEntries());
	
	cout <<"The tree has "<<nEvents<<" Events"<<endl;
	for (size_t i=0;i<nEvents;i++) 
	{
		cout << "reading Event "<< i+1 <<endl;
		T->GetEntry(i);
		if (!ew) ew = new MyEventViewer(event,"Reader");
		drawEvent(event,ew);
	}
}


//___________________________________________________________________________________________________________________________________________________________
void TestRootFile()
{
	//compile the classes with ACLIC//
	gROOT->ProcessLine(".L ./MySignalTrace/MySignalTrace.cpp+");	//creates a signal trace
	gROOT->ProcessLine(".L ./MyPuls/MyPuls.cpp+");					//stores a subset of that trace
	gROOT->ProcessLine(".L ./MyChannel/MyChannel.cpp+");			//a Channel of the event
	gROOT->ProcessLine(".L ./MyEvent/MyEvent.cpp+");				//the Event
	gROOT->ProcessLine(".L ./MyZeroSuppressor/MyZeroSuppressor.cpp+");//suppresses the zeros from the trace
	gROOT->ProcessLine(".L ./MyEventViewer/MyEventViewer.cpp+");	//to view the event

	MyEventViewer * eww=0; //viewer for the written Data
	MyEventViewer * ewr=0; //viewer for the read Data

	writeRootFile(eww);
	readRootFile(ewr);
}

